import { OnInit, TemplateRef, EventEmitter, AfterViewInit, OnDestroy } from '@angular/core';
import { DataGridModel, Direction } from './models/datagrid.model';
import { DatagridStyles } from './models/datagridstyle.model';
/**
 * The datagrid provides basic functions for data tables:
 * * sorting
 * * filtering
 * * pagination
 * * editing
 *
 * Provide an decorator enhanced model and the grid appears driven by model meta data.
 *
 * There are many attributes and ways to change the appearance. Also some classes can be controlled by
 * the host component:
 * * `.col-borders`
 * * `.col-last`
 * * `.col-first`
 * All these styles are applied to the <col> elements of the underlying table.
 */
export declare class DataGridComponent implements OnInit, AfterViewInit, OnDestroy {
    directionEnumHelper: typeof Direction;
    stringFallback: TemplateRef<any>;
    string: TemplateRef<any>;
    booleanFallback: TemplateRef<any>;
    boolean: TemplateRef<any>;
    dateFallback: TemplateRef<any>;
    date: TemplateRef<any>;
    enumFallback: TemplateRef<any>;
    enum: TemplateRef<any>;
    numberFallback: TemplateRef<any>;
    number: TemplateRef<any>;
    externals: {
        [key: string]: any;
    };
    /**
     * Override the internal styles by giving directly CSS rules based on the column tags.
     */
    columnStyle: DatagridStyles;
    /**
     * The model that makes up the content. Shall provide properties with decorators to control appearance.
     */
    model: DataGridModel<any>;
    /**
     * Wheather to show a delete button. Clicking it fires the @see DataGridModel.OnDelete event.
     */
    showDeleteButton: boolean;
    /**
     * Wheather to show an edit button. Clicking it fires the @see DataGridModel.OnEdit event.
     */
    showEditButton: boolean;
    /**
     * Show the action column at all. Use @see showDeleteButton and  @see showEditButton to switch the buttons
     * on or off individually. Default is `true` (actions visible).
     */
    showActions: boolean;
    /**
     * The text that appears on the Delete button. Default is 'Delete'.
     */
    textDeleteButton: string;
    /**
     * The text that appears on the Edit button. Default is 'Edit'.
     */
    textEditButton: string;
    /**
     * The column header of the column that shows the buttons. Default is 'Actions'.
     */
    textButtonsHeader: string;
    /**
     * The text that appears if there are no items to show. Can also be overwritten by a more complex piece
     * of code by adding a template like this:
     * @example
     * <ng-template #data-warning-noitems>
     *   <div class="alert alert-danger">The grid is empty</div>
     * </ng-template>
     */
    textNoItems: string;
    /**
     * The filter value to filter the content. The data is of type
     *
     * @example
     * { [prop: string]: any }
     *
     *
     */
    filter: {
        [prop: string]: any;
    };
    /**
     * If `true` the columns can be rearranged by moving around with drag 'n drop.
     */
    reArrangeColumns: boolean;
    /**
     * Event forwarded from model class and being fired after the model class's onEdit event.
     * The event is invoked by the appropriate internal button via click.
     */
    editItem: EventEmitter<any>;
    /**
     * Event forwarded from model class and being fired after the model class's onDelete event.
     * The event is invoked by the appropriate internal button via click.
     */
    deleteItem: EventEmitter<any>;
    ngOnInit(): void;
    ngAfterViewInit(): void;
    ngOnDestroy(): void;
    private warnProp;
    /**
     * Controls the template used to display certain data types.
     * If the host provides a template it's being used, otherwise a fallback is provided
     * @param uiHint Property of @UiHint decorator
     */
    getActiveTemplate(uiHint: string, prop?: string): TemplateRef<any>;
}
